<?php

namespace Teknisa\SM\Service;

use Teknisa\SM\Factory\Sm00004000DocumentoContratoFactory as DocumentoFactory;
use Teknisa\SM\Util\Utilities;
use Teknisa\SM\Util\PHPWord\SMTemplateProcessor;
use Teknisa\SM\Service\SmDocumentosService;

class Sm00004000DocumentoContratoService {
    
    protected $documentoFactory;
    protected $fileServerBaseUrl;
    protected $smdocumentos;
    
    private $tempDir;
    
    public function __construct($fileServerBaseUrl, DocumentoFactory $documentoFactory, SmDocumentosService $smdocumentos) {
        $this->documentoFactory  = $documentoFactory;
        $this->fileServerBaseUrl = $fileServerBaseUrl;
        $this->smdocumentos      = $smdocumentos;
        
        $this->tempDir = sys_get_temp_dir();
    }
    
    /**
     * Gera a mensagem de erro quando não forem encontratos documentos
    **/
    public function getErrorMessage($idSituacao) {
        switch($idSituacao) {
            case "P": 
                $idSituacao = "Pendente";
                break;
            case "A": 
                $idSituacao = "Aprovado";
                break;
            case "R": 
                $idSituacao = "Reprovado";
                break;
            case "E": 
                $idSituacao = "Encerrado";
                break;
        }
        
        return "Operação bloqueada.<br/>Esse contrato não possui documento cadastrado para a situação '". $idSituacao ."'.";
    }
    
    public function gerarDocumentoContrato($nrseqcontrato, $nrorg, $idSituacao, $idpermitedocx) {
        try {
            $nomeArquivoNovo = null;
            $ds = DIRECTORY_SEPARATOR;
            
            $nomeArquivos = array();
                
            $documentosAux = $this->getDocumentosContrato($nrseqcontrato, $nrorg, $idSituacao);
            
            $documentos = array();
            
            foreach ($documentosAux as $docAux) {
                if (count($documentos) == 0) {
                    $documentos[0] = $docAux;
                } else if ($documentos[0]["IDSITUACAOCONTRATO"] == null && $docAux["IDSITUACAOCONTRATO"] != null) {
                    $documentos[0] = $docAux;
                }
            }

            if ((count($documentosAux)) <= 0) 
                throw new \Exception($this->getErrorMessage($idSituacao));
            
            foreach ($documentos as $indiceDoc => $doc) {
                $microtime = microtime(true);
                $microtime = str_replace(".", "_", $microtime);
                
                if ($doc["DSTIPO"] == "application/pdf"){
                    if(!in_array($doc["DSCAMINHOARQ"], $nomeArquivos)) {
                        $fileName = $microtime . "_" . $doc["NMARQUIVO"];
                        $caminhoFisicoArquivo = $this->smdocumentos->downloadDocFileServer($this->fileServerBaseUrl . $doc["DSCAMINHOARQ"], $fileName, $nrorg);

                        $nomeArquivos[$doc["NRSEQCONTDOC"]] = array(
                            "CAMINHO" => "/downloadArquivo?requestType=Row&row[FILENAME]=" . urlencode(base64_encode($fileName)) . "&row[FILETYPE]=" . $doc["DSTIPO"],
                            "DADOSDOCUMENTO" => array(
                                "NMARQUIVO"    => $doc["NMARQUIVO"],
                                "DSCAMINHOARQ" => $caminhoFisicoArquivo,
                                "TYPE"         => $doc["DSTIPO"]
                            )
                        );
                    }
                } else {
                    if (!empty($doc["DSCAMINHOARQ"])) {
                        $fileName = $microtime . "_" . $doc["NMARQUIVO"];
                        $nomeArquivoDocx = $this->smdocumentos->downloadDocFileServer($this->fileServerBaseUrl . $doc["DSCAMINHOARQ"], $fileName, $nrorg);

                        if (is_file($nomeArquivoDocx)) {
                            $templateProcessor = new SMTemplateProcessor($nomeArquivoDocx);
                            
                            //pega os dados principais do contrato
                            $contrato = $this->getDadosContrato($nrseqcontrato, $nrorg);
                            
                            $nfExp = $this->getNotaExp($nrseqcontrato, $nrorg);
                            
                            //substitui os dados do contrato no documento
                            $this->substituirDadosContratoDocumento($contrato, $nfExp, $templateProcessor);
                            
                            $caminhoLogotipo = $this->getCaminhoLogotipoParceiro($contrato["NRPARCNEGOCIO"], $nrorg);
                            if (!empty($caminhoLogotipo)) {
                                $this->substituirLogotipoDocumento($caminhoLogotipo, $templateProcessor, $doc["VRTAMANHOLOGO"]);
                            }
                            else {
                                $templateProcessor->setValue("[LOGOTIPO]", "");
                            }
                            
                            //busca os dados do comprador do contrato
                            $comprador = $this->getDadosCompradorContrato($contrato["NRPARCNEGOCIO"], $nrorg);
                            
                            //substitui os dados do comprador do contrato no documento
                            $this->substituirDadosCompradorContratoDocumento($comprador, $templateProcessor);
                            
                            //retorna o endereço principal do parceiro (cliente) do contrato
                            $enderecoPrincipal = $this->getEnderecoParceiro($contrato["NRPARCNEGOCIO"], $nrorg, "PRINCIPAL");
                            
                            //substitui o endereço principal do comprador do contrato no documento
                            $this->substituirDadosEnderecoPrincipalCompradorContratoDocumento($enderecoPrincipal, $templateProcessor);
                            
                            //retorna o endereço de entrega da proposta geradora do contrato
                            $enderecoEntrega = $this->getEnderecoEntregaProposta($nrseqcontrato, $nrorg);
                            
                            //substitui os dados do endereço de entrega da proposta no documento
                            $this->substituirDadosEnderecoEntregaPropostaContratoDocumento($enderecoEntrega, $templateProcessor);
                            
                            //retorna os dados do prestador do contrato
                            $dadosPrestador = $this->getDadosPrestadorContrato($contrato["CDFILIALORCA"], $nrorg);
                            
                            //substitui os dados do prestador do contrato no documento
                            $this->substituirDadosPrestadorContratoDocumento($dadosPrestador, $templateProcessor);
                            
                            $documento = $templateProcessor->getTempDocumentMainPart();
                            
                            $gerarDocumento = $this->validarExistenciaItensDocumento($documento, $nrseqcontrato, $nrorg);
                            
                            if(!$gerarDocumento) {
                                $nomeArquivos[$doc["NRSEQCONTDOC"]] = array(
                                    "CAMINHO" => null,
                                    "DADOSDOCUMENTO" => array (
                                        "NMARQUIVO" => null,
                                        "DSCAMINHOARQ" => null,
                                        "TYPE"         => $doc["DSTIPO"]
                                    )
                                );
                            } else {
                                //monta os mnemônicos com índice da parte de itens e adicionais no documento, para serem substituidos pelos valores depois
                                $documento = $this->gerarDocumentoParteDeItens($nrseqcontrato, $nrorg, $documento);
                                
                                //substitui os dados das Assinaturas do contrato.
                                $documento = $this->gerarDocumentoAssinaturas($nrseqcontrato, $nrorg, $documento);
                                
                                //monta os mnemônicos com índice da parte de itens de apuração de faturamento no documento, para serem substituidos pelos valores depois
                                $documento = $this->gerarDocumentoParteDeItensApuracaoFaturamento($nrseqcontrato, $nrorg, $documento);
                                
                                //monta os mnemônicos com índice da parte de itens de modalidade de pagamento no documento, para serem substituidos pelos valores depois
                                $documento = $this->gerarDocumentoParteDeItensModalidadePagamento($nrseqcontrato, $nrorg, $documento);
                                
                                //Monta a parte de itens formação de preço do contrato
                                $documento = $this->gerarDocumentoParteFormacaoPreco($nrorg, $nrseqcontrato, $documento);
                                
                                //monta os mnemônicos com índice da parte de Hora Extra, para serem substituidos pelos valores depois
                                $documento = $this->gerarDocumentoParteDeHoraExtra($nrseqcontrato, $nrorg, $documento);
                                
                                //Substitui os dados de alteração contratual
                                $documento = $this->substituirDadosAlteracaoContratual($nrseqcontrato, $nrorg, $documento);
                                
                                $valorTotalItens = 0;
                                $valorTotalPeriodo = 0;
                                $valorTotalTodosAdicionais = 0;
                                
                                $valorTotalItensApuracaoFaturamento = 0;
                                $valorTotalPeriodoApuracaoFaturamento = 0;
                                
                                $valorTotalItensModalidadePagamento = 0;
                                $valorTotalPeriodoModalidadePagamento = 0;
                                
                                $itens = $this->getItensContrato($nrseqcontrato, $nrorg);
                                foreach ($itens as $item) {
                                    //substitui os dados dos itens do contrato no documento
                                    $documento = $this->substituirDadosItensDocumento($item, $documento);
                            
                                    if ($item["IDFATURACONTR"] == "A") {
                                        $documento = $this->substituirDadosItensApuracaoFaturamentoDocumento($item, $documento);
                                        
                                        $valorTotalItensApuracaoFaturamento += (float)$item["VRTOTITEMCONT"];
                                        $valorTotalPeriodoApuracaoFaturamento += (float)$item["TOTPERIODO"];
                                    }
                                    else if ($item["IDFATURACONTR"] == "O") {
                                        $documento = $this->substituirDadosItensModalidadePagamentoDocumento($item, $documento);
                                        
                                        $valorTotalItensModalidadePagamento += (float)$item["VRTOTITEMCONT"];
                                        $valorTotalPeriodoModalidadePagamento += (float)$item["TOTPERIODO"];
                                    }
                                    
                                    $valorTotalAdicionaisDoItem = 0;
                                    
                                    $itensAdicionais = $this->getItensAdicionaisContrato($nrseqcontrato, $item["NRITEMOCONTRATO"], $nrorg);
                                    foreach ($itensAdicionais as $itemAdicional) {
                                        //substitui os dados dos itens adicionais da proposta no documento
                                        $documento = $this->substituirDadosItensAdicionaisDocumento($item, $itemAdicional, $documento);
                                        
                                        $valorTotalAdicionaisDoItem += (float)$itemAdicional["VRTOTITEMCONT"];
                                    }
                                    
                                    $valorTotalTodosAdicionais += $valorTotalAdicionaisDoItem;
                                    $documento = str_replace("[TOTITEMAD" . $item["NRITEMOCONTRATO"] . "1]", Utilities::formatarNumero($valorTotalAdicionaisDoItem), $documento);
                                    
                                    $valorTotalItens += (float)$item["VRTOTITEMCONT"];
                                    $valorTotalPeriodo += (float)$item["TOTPERIODO"];
                                    
                                    /* Substitui os dados de Hora Extra do Documento */
                                    $documento = $this->substituirDadosHrExtraDocumento($item, $documento);
                                }
                                
                                
                                $valorTotalTodosAdicionais = Utilities::formatarNumero($valorTotalTodosAdicionais);
                                $valorTotalItens = Utilities::formatarNumero($valorTotalItens);
                                $valorTotalPeriodo = Utilities::formatarNumero($valorTotalPeriodo);
                                
                                $valorTotalItensApuracaoFaturamento = Utilities::formatarNumero($valorTotalItensApuracaoFaturamento);
                                $valorTotalPeriodoApuracaoFaturamento = Utilities::formatarNumero($valorTotalPeriodoApuracaoFaturamento);
                                
                                $valorTotalItensModalidadePagamento = Utilities::formatarNumero($valorTotalItensModalidadePagamento);
                                $valorTotalPeriodoModalidadePagamento = Utilities::formatarNumero($valorTotalPeriodoModalidadePagamento);
                                
                                //$totcont = $this->getValorTotalContrato($nrseqcontrato, $nrorg);
                                //$totcont = Utilities::formatarNumero($totcont);
                                
                                $documento = str_replace("[TOTGERAD]", $valorTotalTodosAdicionais, $documento);
                                
                                $documento = str_replace("[TOTITEM]", $valorTotalItens, $documento);
                                $documento = str_replace("[TOTPERIT]", $valorTotalPeriodo, $documento);
                                
                                $documento = str_replace("[ATOTITEM]", $valorTotalItensApuracaoFaturamento, $documento);
                                $documento = str_replace("[ATOTPERIT]", $valorTotalPeriodoApuracaoFaturamento, $documento);
                                
                                $documento = str_replace("[OTOTITEM]", $valorTotalItensModalidadePagamento, $documento);
                                $documento = str_replace("[OTOTPERIT]", $valorTotalPeriodoModalidadePagamento, $documento);
                                
                                //$documento = str_replace("[TOTCONT]", $totorca, $totcont);
                                
                                $documento = str_replace("[LN]", "", $documento);
                                $documento = str_replace("[/LN]", "", $documento);
                                
                                $templateProcessor->setTempDocumentMainPart($documento);
                                
                                $fileName = $microtime . "_" . str_replace(array("/", "\\"), "_", $contrato["NRCONTRATOFCT"]) . "_" . $indiceDoc . ".docx";
                                $fileName = str_replace(",", "_", $fileName);
                                $nomeArquivoNovo = $this->tempDir . $ds . $fileName;
                                
                                $templateProcessor->saveAs($nomeArquivoNovo);
                                
                                if (!is_file($nomeArquivoNovo)) {
                                    throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoNovo."\" não foi encontrado.");
                                }
                                
                                if($idpermitedocx == "D"){
                                    //gerando docx
                                    $nomeArquivos[$doc["NRSEQCONTDOC"]] = array(
                                        "CAMINHO" => "/downloadArquivo?requestType=Row&row[FILENAME]=" . urlencode(base64_encode($fileName)) . "&row[FILETYPE]=application/vnd.openxmlformats-officedocument.wordprocessingml.document",
                                        "DADOSDOCUMENTO" => array (
                                            "NMARQUIVO" => $fileName,
                                            "DSCAMINHOARQ" => $this->tempDir . $ds . $fileName,
                                            "TYPE"         => $doc["DSTIPO"]
                                        )
                                    );
                                } else {
                                    //gerando pdf
                                    $cmd = "export HOME=" .$this->tempDir. " && soffice --headless --convert-to pdf --outdir " .$this->tempDir. " \"" .$nomeArquivoNovo. "\" 2>&1";
                                    $resultadoCmd = shell_exec($cmd);
                                    
                                    if (strpos(strtolower($resultadoCmd), "error") !== false) {
                                            throw new \Exception("Falha ao gerar o arquivo PDF.<br/>Erro: ".$resultadoCmd);
                                    } else {
                                        if (is_file($nomeArquivoNovo)) {
                                            unlink($nomeArquivoNovo);
                                        }
                                        
                                        if (is_file($caminhoLogotipo)) {
                                            unlink($caminhoLogotipo);
                                        }
                                        
                                        $fileName = $microtime . "_" . str_replace(array("/", "\\"), "_", $contrato["NRCONTRATOFCT"]) . "_" . $indiceDoc . ".pdf";
                                        $fileName = str_replace(",", "_", $fileName);
                                        $nomeArquivoNovo = $this->tempDir . $ds . $fileName;
                                        
                                        if (is_file($nomeArquivoNovo)) {
                                            $nomeArquivos[$doc["NRSEQCONTDOC"]] = array(
                                                "CAMINHO" => "/downloadArquivo?requestType=Row&row[FILENAME]=" .  urlencode(base64_encode($fileName)) . "&row[FILETYPE]=application/pdf",
                                                "DADOSDOCUMENTO" => array (
                                                    "NMARQUIVO" => $fileName,
                                                    "DSCAMINHOARQ" => $this->tempDir . $ds . $fileName,
                                                    "TYPE"         => $doc["DSTIPO"]
                                                )
                                            );
                                        } else {
                                            throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoNovo."\" não foi encontrado.<br/>Erro: ".$resultadoCmd);
                                        }
                                    }
                                }
                            }
                        } else {
                            throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoDocx."\" não foi encontrado.");
                        }
                        
                        if (is_file($nomeArquivoDocx)) {
                            unlink($nomeArquivoDocx);
                        }
                    }
                }
            }
            return $nomeArquivos;
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    

    public function gerarDocumentoContratoRescisao($contratos, $nrTipoRescisao, $nrorg, $idSituacao, $idExtensao) {
        try {
            
            $arquivos = array();
            $parcneg = null;
            
            foreach ($contratos as $indice => $contrato) {
                $listaContr = null;
                
                foreach($contrato as $cont){
                    
                    $listaContr = $listaContr.$cont.', ';
                }
                
                $listaContr = substr($listaContr,0, strlen($listaContr)-2);
                
                foreach($contrato as $cont){
                    
                    
                    $nrseqcontrato = $cont;
                    
                    if ($parcneg != $indice) {
                        $nomeArquivoNovo = null;
                        $ds = DIRECTORY_SEPARATOR;
                        
                        $nomeArquivos = array();
                        
                        $documentosAux = $this->getDocumentosRescisao($nrseqcontrato, $nrorg, $nrTipoRescisao);
                        
                        $documentos = array();
                        
                        foreach ($documentosAux as $docAux) {
                            if (count($documentos) == 0) {
                                $documentos[0] = $docAux;
                            } else if ($documentos[0]["IDSITUACAOCONTRATO"] == null && $docAux["IDSITUACAOCONTRATO"] != null) {
                                $documentos[0] = $docAux;
                            }
                        }
            
                        if ((count($documentosAux)) <= 0) 
                            throw new \Exception($this->getErrorMessage($idSituacao));
                        
                        foreach ($documentos as $indiceDoc => $doc) {
                            $microtime = microtime(true);
                            $microtime = str_replace(".", "_", $microtime);
                            
                            if ($doc["DSTIPO"] == "application/pdf"){
                                if(!in_array($doc["DSCAMINHOARQ"], $nomeArquivos)) {
                                    $fileName = $microtime . "_" . $doc["NMARQUIVO"];
                                    $caminhoFisicoArquivo = $this->smdocumentos->downloadDocFileServer($this->fileServerBaseUrl . $doc["DSCAMINHOARQ"], $fileName, $nrorg);
            
                                    $nomeArquivos[$doc["NRSEQMODDOCFCT"]] = array(
                                        "CAMINHO" => "/downloadArquivo?requestType=Row&row[FILENAME]=" . urlencode(base64_encode($fileName)) . "&row[FILETYPE]=" . $doc["DSTIPO"],
                                        "DADOSDOCUMENTO" => array(
                                            "NMARQUIVO"    => $doc["NMARQUIVO"],
                                            "DSCAMINHOARQ" => $caminhoFisicoArquivo,
                                            "TYPE"         => $doc["DSTIPO"]
                                        )
                                    );
                                }
                            } else {
                                if (!empty($doc["DSCAMINHOARQ"])) {
                                    $fileName = $microtime . "_" . $doc["NMARQUIVO"];
                                    $nomeArquivoDocx = $this->smdocumentos->downloadDocFileServer($this->fileServerBaseUrl . $doc["DSCAMINHOARQ"], $fileName, $nrorg);
            
                                    if (is_file($nomeArquivoDocx)) {
                                        $templateProcessor = new SMTemplateProcessor($nomeArquivoDocx);
                                        
                                        //pega os dados principais do contrato
                                        $contrato = $this->getDadosContrato($nrseqcontrato, $nrorg);
                                        
                                        $nfExp = $this->getNotaExp($nrseqcontrato, $nrorg);
                                        
                                        //substitui os dados do contrato no documento
                                        $this->substituirDadosContratoDocumento($contrato, $nfExp, $templateProcessor);
                                        
                                        $caminhoLogotipo = $this->getCaminhoLogotipoParceiro($contrato["NRPARCNEGOCIO"], $nrorg);
                                        if (!empty($caminhoLogotipo)) {
                                            $this->substituirLogotipoDocumento($caminhoLogotipo, $templateProcessor, $doc["VRTAMANHOLOGO"]);
                                        }
                                        else {
                                            $templateProcessor->setValue("[LOGOTIPO]", "");
                                        }
                                        
                                        //busca os dados do comprador do contrato
                                        $comprador = $this->getDadosCompradorContrato($contrato["NRPARCNEGOCIO"], $nrorg);
                                        
                                        //substitui os dados do comprador do contrato no documento
                                        $this->substituirDadosCompradorContratoDocumento($comprador, $templateProcessor);
                                        
                                        //retorna o endereço principal do parceiro (cliente) do contrato
                                        $enderecoPrincipal = $this->getEnderecoParceiro($contrato["NRPARCNEGOCIO"], $nrorg, "PRINCIPAL");
                                        
                                        //substitui o endereço principal do comprador do contrato no documento
                                        $this->substituirDadosEnderecoPrincipalCompradorContratoDocumento($enderecoPrincipal, $templateProcessor);
                                        
                                        //retorna o endereço de entrega da proposta geradora do contrato
                                        $enderecoEntrega = $this->getEnderecoEntregaProposta($nrseqcontrato, $nrorg);
                                        
                                        //substitui os dados do endereço de entrega da proposta no documento
                                        $this->substituirDadosEnderecoEntregaPropostaContratoDocumento($enderecoEntrega, $templateProcessor);
                                        
                                        //retorna os dados do prestador do contrato
                                        $dadosPrestador = $this->getDadosPrestadorContrato($contrato["CDFILIALORCA"], $nrorg);
                                        
                                        //substitui os dados do prestador do contrato no documento
                                        $this->substituirDadosPrestadorContratoDocumento($dadosPrestador, $templateProcessor);
                                        
                                        $documento = $templateProcessor->getTempDocumentMainPart();
                                        
                                        //substitui os dados das Assinaturas do contrato.
                                        $documento = $this->gerarDocumentoAssinaturas($nrseqcontrato, $nrorg, $documento);
                                        
                                        //Substitui os dados de Rescisão do contrato.
                                        $documento = $this->gerarDocumentoRescisaoContr($nrseqcontrato, $nrorg, $documento, $listaContr);
                                        
                                        $documento = $this->substituirDadosContratosRescindidos($nrseqcontrato, $nrorg, $documento, $listaContr);
                                        
                                        //Substitui os dados de Rescisão do contrato.
                                        $documento = $this->gerarDocumentoDadosRescisao($nrseqcontrato, $nrorg, $documento);
                                        
                                        $documento = str_replace("[LN]", "", $documento);
                                        $documento = str_replace("[/LN]", "", $documento);
                                
                                        $templateProcessor->setTempDocumentMainPart($documento);
                                        
                                        $fileName = $microtime . "_" . str_replace(array("/", "\\"), "_", $contrato["NRCONTRATOFCT"]) . "_" . $indiceDoc . ".docx";
                                        $fileName = str_replace(",", "_", $fileName);
                                        $nomeArquivoNovo = $this->tempDir . $ds . $fileName;
                                        
                                        $templateProcessor->saveAs($nomeArquivoNovo);
                                        
                                        if (!is_file($nomeArquivoNovo)) {
                                            throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoNovo."\" não foi encontrado.");
                                        }
                                        
                                        if($idExtensao == "D"){
                                            //gerando docx
                                            $nomeArquivos[$doc["NRSEQMODDOCFCT"]] = array(
                                                "CAMINHO" => "/downloadArquivo?requestType=Row&row[FILENAME]=" . urlencode(base64_encode($fileName)) . "&row[FILETYPE]=application/vnd.openxmlformats-officedocument.wordprocessingml.document",
                                                "DADOSDOCUMENTO" => array (
                                                    "NMARQUIVO" => $fileName,
                                                    "DSCAMINHOARQ" => $this->tempDir . $ds . $fileName,
                                                    "TYPE"         => "application/vnd.openxmlformats-officedocument.wordprocessingml.document"
                                                )
                                            );
                                        } else {
                                            //gerando pdf
                                            $cmd = "export HOME=" .$this->tempDir. " && soffice --headless --convert-to pdf --outdir " .$this->tempDir. " \"" .$nomeArquivoNovo. "\" 2>&1";
                                            $resultadoCmd = shell_exec($cmd);
                                            
                                            if (strpos(strtolower($resultadoCmd), "error") !== false) {
                                                    throw new \Exception("Falha ao gerar o arquivo PDF.<br/>Erro: ".$resultadoCmd);
                                            } else {
                                                if (is_file($nomeArquivoNovo)) {
                                                    unlink($nomeArquivoNovo);
                                                }
                                                
                                                if (is_file($caminhoLogotipo)) {
                                                    unlink($caminhoLogotipo);
                                                }
                                                
                                                $fileName = $microtime . "_" . str_replace(array("/", "\\"), "_", $contrato["NRCONTRATOFCT"]) . "_" . $indiceDoc . ".pdf";
                                                $fileName = str_replace(",", "_", $fileName);
                                                $nomeArquivoNovo = $this->tempDir . $ds . $fileName;
                                                
                                                if (is_file($nomeArquivoNovo)) {
                                                    $nomeArquivos[$doc["NRSEQMODDOCFCT"]] = array(
                                                        "CAMINHO" => "/downloadArquivo?requestType=Row&row[FILENAME]=" .  urlencode(base64_encode($fileName)) . "&row[FILETYPE]=application/pdf",
                                                        "DADOSDOCUMENTO" => array (
                                                            "NMARQUIVO" => $fileName,
                                                            "DSCAMINHOARQ" => $this->tempDir . $ds . $fileName,
                                                            "TYPE"         => "application/pdf"
                                                        )
                                                    );
                                                } else {
                                                    throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoNovo."\" não foi encontrado.<br/>Erro: ".$resultadoCmd);
                                                }
                                            }
                                        }
                                    }
                                } else {
                                    throw new \Exception("Falha ao gerar o documento. O arquivo \"".$nomeArquivoDocx."\" não foi encontrado.");
                                }
                                
                                if (is_file($nomeArquivoDocx)) {
                                    unlink($nomeArquivoDocx);
                                }
                            }
                        }
                        
                        array_push($arquivos, $nomeArquivos);
                        $parcneg = $indice;
                    }
                }    
            }
            
            return $arquivos;
        }
        catch (\Exception $e) {
            throw $e;
        }
    }
    
    
    /**
     * Retorna o documentos do contrato para o escopo informado.
     */
    private function getDocumentosContrato($nrseqcontrato, $nrorg, $idSituacao) {
        $params = array(
            "NRSEQCONTRATO"      => $nrseqcontrato,
            "NRORG"              => $nrorg,
            "IDSITUACAOCONTRATO" => $idSituacao
        );
        
        return $this->documentoFactory->getDocumentosContrato($params);
    }
    
    /**
     * Retorna o documentos de rescisão do contrato para o escopo informado.
     */
    private function getDocumentosRescisao($nrseqcontrato, $nrorg, $nrTipoRescisao) {
        $params = array(
            "NRSEQCONTRATO"  => $nrseqcontrato,
            "NRTIPORESCISAO" => $nrTipoRescisao,
            "NRORG"          => $nrorg
        );
        
        return $this->documentoFactory->getDocumentosRescisao($params);
    }
    
    
    /**
     * Retorna os dados principais do contrato
     */
    private function getDadosContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->documentoFactory->getDadosContrato($params);
        return $dataset[0];
    }
    
    private function getNotaExp($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->documentoFactory->getNotaExp($params);
        return $dataset[0]["NRNOTAFISC"];
    }
    
    /**
     * Substitui os dados do contrato no documento.
     */
    private function substituirDadosContratoDocumento($contrato, $nfExp, $templateProcessor) {
        $dataAtual = date_create();
        $nomeMes = Utilities::getNomeMesPortugues($dataAtual);
        
        $templateProcessor->setValue("[CDCONT]", $contrato["NRCONTRATOFCT"]);
        $templateProcessor->setValue("[DTINI]", $contrato["DTINICIOCONTRATO"]);
        $templateProcessor->setValue("[DTFIN]", $contrato["DTFIMCONTRATO"]);
        $templateProcessor->setValue("[PRAZO]", $contrato["NRDIASPRAZOCONT"]);
        $templateProcessor->setValue("[INDFIN]", $contrato["CDINDFINCONT"]);
        $templateProcessor->setValue("[DTREA]", $contrato["DTPADREAJCONT"]);
        $templateProcessor->setValue("[QTPOST]", $contrato["QTPOSTOTRABCONT"]);
        $templateProcessor->setValue("[DTPRORAUT]", $contrato["DTPADRPRORGCONT"]);
        $templateProcessor->setValue("[INTCON]", $contrato["NMPARCINTERLOCUTOR"]);
        $templateProcessor->setValue("[MOPGCON]", $contrato["DSMODPGTO"]);
        $templateProcessor->setValue("[DTVENPARC]", ""); // ??
        $templateProcessor->setValue("[DTHOJE]", $dataAtual->format("d/m/Y"));
        $templateProcessor->setValue("[DIAAT]", $dataAtual->format("d"));
        $templateProcessor->setValue("[MESAT]", $dataAtual->format("m"));
        $templateProcessor->setValue("[ANOAT]", $dataAtual->format("Y"));
        $templateProcessor->setValue("[NMMESAT]", $nomeMes);
        $templateProcessor->setValue("[REFCONT]", $contrato["DSREFERENCIACONT"]);
        $templateProcessor->setValue("[OBSCONT]", $contrato["DSOBSERVACAOCONT"]);
        $templateProcessor->setValue("[TOTREC]", $contrato["VRRECUCONT"]);
        $templateProcessor->setValue("[NFEXP]", $nfExp);
        $templateProcessor->setValue("[OPRESPCONT]", $contrato["NMOPERRESPONSAVEL"]);
        $templateProcessor->setValue("[PRAZOMES]", $contrato["PRAZOMES"]);
    }
    
    /**
     * Busca os dados do comprador do contrato.
     */
    private function getDadosCompradorContrato($nrparcnegocio, $nrorg) {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->documentoFactory->getDadosCompradorContrato($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Substitui os dados do contrato no documento.
     */
    private function substituirDadosCompradorContratoDocumento($comprador, $templateProcessor) {
        $inscricao = Utilities::formatarCpfCnpj($comprador["NRINSCRICAOPARC"]);
        
        $templateProcessor->setValue("[CPARC]", $inscricao);
        $templateProcessor->setValue("[CNMPARC]", $comprador["NMPRINCIPALPARC"]);
        $templateProcessor->setValue("[CIE]", $comprador["NRINSESTCLIE"]);
    }
    
    /**
     * Busca o endereço de um determinado tipo do parceiro de negócio informado.
     */
    public function getEnderecoParceiro($nrparcnegocio, $nrorg, $cdtipoendereco) {
        $params = array(
            "NRPARCNEGOCIO"  => $nrparcnegocio,
            "NRORG"          => $nrorg,
            "CDTIPOENDERECO" => $cdtipoendereco
        );
        
        $dataset = $this->documentoFactory->getEnderecoParceiro($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Substitui o endereço principal do comprador do contrato no documento.
     */
    public function substituirDadosEnderecoPrincipalCompradorContratoDocumento($endereco, $templateProcessor) {
        if (!empty($endereco)) {
            $cep = Utilities::formatarCep($endereco["NRCEPENDERECO"]);
            
            $templateProcessor->setValue("[CENDE]", $endereco["ENDERECO"]);
            $templateProcessor->setValue("[CDSEND]", $endereco["DSENDERECO"]);
            $templateProcessor->setValue("[CMUNIC]", $endereco["NMMUNICIPIO"]);
            $templateProcessor->setValue("[CBAIRRO]", $endereco["NMBAIRROENDERECO"]);
            $templateProcessor->setValue("[CESTADO]", $endereco["SGESTADO"]);
            $templateProcessor->setValue("[CNMESTADO]", $endereco["NMESTADO"]);
            $templateProcessor->setValue("[CCEP]", $cep);
        }
        else {
            $mnemonicos = array("[CENDE]", "[CDSEND]", "[CMUNIC]", "[CBAIRRO]", "[CESTADO]", "[CNMESTADO]", "[CCEP]");
            $templateProcessor->setValue($mnemonicos, "");
        }
    }
    
    /**
     * Retorna o endereço de entrega da proposta geradora do contrato.
     */
    public function getEnderecoEntregaProposta($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->documentoFactory->getEnderecoEntregaProposta($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Substitui os dados do endereço de entrega da proposta geradora do contrato no documento.
     */
    public function substituirDadosEnderecoEntregaPropostaContratoDocumento($enderecoEntrega, $templateProcessor) {
        if (!empty($enderecoEntrega)) {
            $cep = Utilities::formatarCep($enderecoEntrega["NRCEPENDERECO"]);
            
            $templateProcessor->setValue("[CENDE2]", $enderecoEntrega["ENDERECO"]);
            $templateProcessor->setValue("[CDSEND2]", $enderecoEntrega["DSENDERECO"]);
            $templateProcessor->setValue("[CMUNIC2]", $enderecoEntrega["NMMUNICIPIO"]);
            $templateProcessor->setValue("[CBAIRRO2]", $enderecoEntrega["NMBAIRROENDERECO"]);
            $templateProcessor->setValue("[CESTADO2]", $enderecoEntrega["SGESTADO"]);
            $templateProcessor->setValue("[CCEP2]", $cep);
        }
        else {
            $mnemonicos = array("[CENDE2]", "[CDSEND2]", "[CMUNIC2]", "[CBAIRRO2]", "[CESTADO2]", "[CCEP2]");
            $templateProcessor->setValue($mnemonicos, "");
        }
    }
    
    /**
     * Retorna os dados do prestador do contrato.
     */
    private function getDadosPrestadorContrato($cdfilial, $nrorg) {
        $params = array(
            "CDFILIAL" => $cdfilial,
            "NRORG"    => $nrorg
        );
        
        $dataset = $this->documentoFactory->getDadosPrestadorContrato($params);
        return (count($dataset) > 0) ? $dataset[0] : null;
    }
    
    /**
     * Substitui os dados do prestador do contrato no documento.
     */
    public function substituirDadosPrestadorContratoDocumento($dadosPrestador, $templateProcessor) {
        
        $inscricao = Utilities::formatarCpfCnpj($dadosPrestador["NRINSJURFILI"]);
        $cep = Utilities::formatarCep($dadosPrestador["NRCEPFILI"]);
        
        $templateProcessor->setValue("[PPARC]", $inscricao);
        $templateProcessor->setValue("[PNMPARC]", $dadosPrestador["NMRAZSOCFILI"]);
        $templateProcessor->setValue("[PIE]", $dadosPrestador["CDINSCESTA"]);
        $templateProcessor->setValue("[PENDE]", $dadosPrestador["ENDERECO"]);
        $templateProcessor->setValue("[PDSEND]", $dadosPrestador["DSENDEFILI"]);
        $templateProcessor->setValue("[PMUNIC]", $dadosPrestador["NMMUNICIPIO"]);
        $templateProcessor->setValue("[PBAIRRO]", $dadosPrestador["NMBAIRFILI"]);
        $templateProcessor->setValue("[PESTADO]", $dadosPrestador["SGESTADO"]);
        $templateProcessor->setValue("[PCEP]", $cep);
    }
    
    /**
     * Monta os mnemônicos com índice da parte de itens e adicionais no documento, para serem substituidos pelos valores depois.
     */
    public function gerarDocumentoParteDeItens($nrseqcontrato, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        $result = null;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[IND]" no documento inteiro
        $posicaoInd = strpos($documento, "[IND]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr" que antecede o "[IND]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
            $posicaoPrimeiroLN = strpos($documento, "[/LN]", $posicaoInd);
            
            //posicao do primeiro "[LN]" que antecede o "[IND]"
            $posicaoFechaLN = strrpos($documento, "[LN]", (($tamanhoDocumento - $posicaoPrimeiroLN) * -1));
            
            
            //verifica se existe mnemonico de item adicional no documento
            $existeMnemonicoAdicional = strpos($documento, "[INDAD]");
            
            
            if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN !== false) {
                if ($posicaoPrimeiroTr < $posicaoPrimeiroLN) {
                    $result = "C";
                
                    if ($existeMnemonicoAdicional !== false) {
                        //posicao do primeiro "</w:tbl>" no documento, contando a partir de $posicaoInd
                        $posFin = strpos($documento, "</w:tbl>", $posicaoInd);
                        
                        //posicao do primeiro "<w:tbl" que antecede o "[IND]"
                        $posIni = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posFin) * -1));
                        
                        $tagFecha = "</w:tbl>";
                    }
                    else {
                        $posIni = $posicaoPrimeiroTr;
                        $posFin = $posicaoFechaTr;
                        $tagFecha = "</w:tr>";
                    }
                }
                else {
                    $result = "L";
                
                    //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                    $posFin = strpos($documento, "[/LN]", $posicaoInd);
                    
                    //posicao do primeiro "[LN]" que antecede o "[IND]"
                    $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                    
                    $tagFecha = "[/LN]";
                }
            }
            else if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN === false) {
                $result = "C";
                
                if ($existeMnemonicoAdicional !== false) {
                    //posicao do primeiro "</w:tbl>" no documento, contando a partir de $posicaoInd
                    $posFin = strpos($documento, "</w:tbl>", $posicaoInd);
                    
                    //posicao do primeiro "<w:tbl" que antecede o "[IND]"
                    $posIni = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posFin) * -1));
                    
                    $tagFecha = "</w:tbl>";
                }
                else {
                    $posIni = $posicaoPrimeiroTr;
                    $posFin = $posicaoFechaTr;
                    $tagFecha = "</w:tr>";
                }
            }
            else {
                $result = "L";
                
                //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                $posFin = strpos($documento, "[/LN]", $posicaoInd);
                
                //posicao do primeiro "[LN]" que antecede o "[IND]"
                $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "[/LN]";
            }
            
            $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
            $clobLinha = substr($documento, $posIni, $posFin - $posIni);
            $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
            
            $linhasCodigoItens = $this->getCodigoItensContrato($nrseqcontrato, $nrorg);
            $qtdItens = count($linhasCodigoItens);
            
            foreach ($linhasCodigoItens as $linhaCodigoItem) {
                //coloca o índice nos mnemônicos dos itens
                $clobLinhaSubstituida = $this->colocarIndiceMnemonicoItens($linhaCodigoItem["IND"], $clobLinha);
                
                //monta a parte de itens adicionais no documento
                $clobLinhaSubstituida = $this->gerarDocumentoParteDeAdicionais($nrseqcontrato, $nrorg, $linhaCodigoItem["NRITEMOCONTRATO"], $clobLinhaSubstituida);
                
                if ($result == "C") {
                    if ($existeMnemonicoAdicional !== false) {
                        $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;//."<w:br/>";
                        //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1||'</w:tbl>';
                    }
                    else {
                        $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                        //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1||'</w:tr>';
                    }
                }
                else {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida;
                    //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1;
                }
            }
            
            if ($qtdItens > 0) {
                //retira os mnemônicos dos itens
                $clobLinhaSubstituida = $this->retirarMnemonicosItens($clobLinha);
                
                $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
            }
            
            $documento = str_replace($clobLinha.$tagFecha, "", $documento);
        }
        
        return $documento;
    }
    
    public function getCodigoItensContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->documentoFactory->getCodigoItensContrato($params);
    }
    
    /**
     * Coloca o índice nos mnemônicos dos itens da proposta.
     */
    private function colocarIndiceMnemonicoItens($codigoItem, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[IND]", "[IND" . $codigoItem . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[COD]", "[COD" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QNT]", "[QNT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESC]", "[DESC" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTMED]", "[QTMED" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[MEDIDA]", "[MEDIDA" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPSERV]", "[TPSERV" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTTPSERV]", "[QTTPSERV" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[MODPAG]", "[MODPAG" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRUNIT]", "[VRUNIT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOT]", "[VRTOT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TOTPER]", "[TOTPER" . $codigoItem . "]", $clobLinhaSubstituida);
        
        $clobLinhaSubstituida = str_replace("[VRDESC]", "[VRDESC" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRACRE]", "[VRACRE" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRAJUSTE]", "[VRAJUSTE" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRSEMDESC]", "[VRSEMDESC" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRSEMAJUSTE]", "[VRSEMAJUSTE" . $codigoItem . "]", $clobLinhaSubstituida);
        
        //$clobLinhaSubstituida = str_replace("[VRPRRECI]", "[VRPRRECI" . $codigoItem . "]", $clobLinhaSubstituida);
        //$clobLinhaSubstituida = str_replace("[VRRECI]", "[VRRECI" . $codigoItem . "]", $clobLinhaSubstituida);
        
        /*$clobLinhaSubstituida = str_replace("[TPHIERQ]", "[TPHIERQ" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OBSERV]", "[OBSERV" . $codigoItem . "]", $clobLinhaSubstituida);*/
        
        return $clobLinhaSubstituida;
    }
    
    private function gerarDocumentoParteDeAdicionais($nrseqcontrato, $nrorg, $nritemocontrato, $clobLinhaSubstituida) {
        $posIniAd = -1;
        $posFinAd = -1;
        
        $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
        
        //posicao do primeiro "[INDAD]" dentro de $clobLinhaSubstituida
        $posicaoIndAd = strpos($clobLinhaSubstituida, "[INDAD]");
        
        if ($posicaoIndAd !== false) {
            //posicao do primeiro "</w:tr>" dentro de $clobLinhaSubstituida, contando a partir de $posicaoIndAd
            $posicaoFechaTrAd = strpos($clobLinhaSubstituida, "</w:tr>", $posicaoIndAd);
            
            //posicao do primeiro "<w:tr" que antecede o "[INDAD]" dentro de $clobLinhaSubstituida
            $posicaoPrimeiroTrAd = strrpos($clobLinhaSubstituida, "<w:tr ", (($tamanhoClobLinhaSubstituida - $posicaoFechaTrAd) * -1));
            
            if (!$posicaoPrimeiroTrAd) {
                $posicaoPrimeiroTrAd = strrpos($clobLinhaSubstituida, "<w:tr>", (($tamanhoClobLinhaSubstituida - $posicaoFechaTrAd) * -1));
            }
            
            if ($posicaoPrimeiroTrAd !== false) {
                $posIniAd = $posicaoPrimeiroTrAd;
                $posFinAd = $posicaoFechaTrAd;
                $tagFecha = "</w:tr>";
                
                $clobInicialAd = substr($clobLinhaSubstituida, 0, $posFinAd + strlen($tagFecha));
                $clobLinhaAd = substr($clobLinhaSubstituida, $posIniAd, $posFinAd - $posIniAd);
                $clobFinalAd = substr($clobLinhaSubstituida, $posFinAd, $tamanhoClobLinhaSubstituida);
                
                $linhasCodigoItensAdicionais = $this->getCodigoItensAdicionaisContrato($nrseqcontrato, $nritemocontrato, $nrorg);
                
                $qtdAdicionais = count($linhasCodigoItensAdicionais);
                
                foreach ($linhasCodigoItensAdicionais as $linhaCodigoItemAdicional) {
                    
                    //coloca o índice nos mnemônicos dos itens adicionais
                    $clobLinhaAdSubstituida = $this->colocarIndiceMnemonicosItensAdicionais($nritemocontrato, $linhaCodigoItemAdicional["IND"], $clobLinhaAd);
                    
                    $clobInicialAd = $clobInicialAd . $clobLinhaAdSubstituida . $tagFecha;
                }
                
                if ($qtdAdicionais > 0) {
                    //retira os mnemônicos dos itens adicionais
                    $clobLinhaAdSubstituida = $this->retirarMnemonicosItensAdicionais($clobLinhaAd);
                    
                    $clobLinhaSubstituida = $clobInicialAd . substr($clobFinalAd, strlen($tagFecha));
                }
                
                $clobLinhaSubstituida = str_replace($clobLinhaAd.$tagFecha, "", $clobLinhaSubstituida);
            }
        }
        
        return $clobLinhaSubstituida;
    }
    
    private function getCodigoItensAdicionaisContrato($nrseqcontrato, $nritemocontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO"   => $nrseqcontrato,
            "NRITEMOCONTRATO" => $nritemocontrato,
            "NRORG"           => $nrorg
        );
        
        return $this->documentoFactory->getCodigoItensAdicionaisContrato($params);
    }
    
    private function colocarIndiceMnemonicosItensAdicionais($nritemocontrato, $indice, $clobLinhaAd) {
        $clobLinhaAdSubstituida = str_replace("[INDAD]", "[INDAD" . $nritemocontrato . $indice . "]", $clobLinhaAd);
        $clobLinhaAdSubstituida = str_replace("[DESCAD]", "[DESCAD" . $nritemocontrato . $indice . "]", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[QNTAD]", "[QNTAD" . $nritemocontrato . $indice . "]", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[VRUNITAD]", "[VRUNITAD" . $nritemocontrato . $indice . "]", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[MEDIDAAD]", "[MEDIDAAD" . $nritemocontrato . $indice . "]", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[QTMEDAD]", "[QTMEDAD" . $nritemocontrato . $indice . "]", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[VRTOTAD]", "[VRTOTAD" . $nritemocontrato . $indice . "]", $clobLinhaAdSubstituida);
        
        return $clobLinhaAdSubstituida;
    }
    
    private function retirarMnemonicosItensAdicionais($clobLinhaAd) {
        $clobLinhaAdSubstituida = str_replace("[INDAD]", "", $clobLinhaAd);
        $clobLinhaAdSubstituida = str_replace("[DESCAD]", "", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[QNTAD]", "", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[VRUNITAD]", "", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[MEDIDAAD]", "", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[QTMEDAD]", "", $clobLinhaAdSubstituida);
        $clobLinhaAdSubstituida = str_replace("[VRTOTAD]", "", $clobLinhaAdSubstituida);
        
        return $clobLinhaAdSubstituida;
    }
    
    public function gerarDocumentoParteDeItensApuracaoFaturamento($nrseqcontrato, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        $result = null;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[AIND]" no documento inteiro
        $posicaoInd = strpos($documento, "[AIND]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr" que antecede o "[AIND]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
            $posicaoPrimeiroLN = strpos($documento, "[/LN]", $posicaoInd);
            
            //posicao do primeiro "[LN]" que antecede o "[AIND]"
            $posicaoFechaLN = strrpos($documento, "[LN]", (($tamanhoDocumento - $posicaoPrimeiroLN) * -1));
            
            
            if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN !== false) {
                if ($posicaoPrimeiroTr < $posicaoPrimeiroLN) {
                    $result = "C";
                    
                    $posIni = $posicaoPrimeiroTr;
                    $posFin = $posicaoFechaTr;
                    $tagFecha = "</w:tr>";
                }
                else {
                    $result = "L";
                
                    //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                    $posFin = strpos($documento, "[/LN]", $posicaoInd);
                    
                    //posicao do primeiro "[LN]" que antecede o "[IND]"
                    $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                    
                    $tagFecha = "[/LN]";
                }
            }
            else if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN === false) {
                $result = "C";
                
                $posIni = $posicaoPrimeiroTr;
                $posFin = $posicaoFechaTr;
                $tagFecha = "</w:tr>";
            }
            else {
                $result = "L";
                
                //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                $posFin = strpos($documento, "[/LN]", $posicaoInd);
                
                //posicao do primeiro "[LN]" que antecede o "[IND]"
                $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "[/LN]";
            }
            
            $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
            $clobLinha = substr($documento, $posIni, $posFin - $posIni);
            $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
            
            $linhasCodigoItens = $this->getCodigoItensApuracaoFaturamentoContrato($nrseqcontrato, $nrorg);
            $qtdItens = count($linhasCodigoItens);
            
            foreach ($linhasCodigoItens as $linhaCodigoItem) {
                //coloca o índice nos mnemônicos dos itens
                $clobLinhaSubstituida = $this->colocarIndiceMnemonicoItensApuracaoFaturamento($linhaCodigoItem["IND"], $clobLinha);
                
                if ($result == "C") {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                    //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1||'</w:tr>';
                }
                else {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida;
                    //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1;
                }
            }
            
            if ($qtdItens > 0) {
                //retira os mnemônicos dos itens
                $clobLinhaSubstituida = $this->retirarMnemonicosItensApuracaoFaturamento($clobLinha);
                
                $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
            }
            
            $documento = str_replace($clobLinha.$tagFecha, "", $documento);
        }
        
        return $documento;
    }
    
    private function colocarIndiceMnemonicoItensApuracaoFaturamento($codigoItem, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[AIND]", "[AIND" . $codigoItem . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[ACOD]", "[ACOD" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AQNT]", "[AQNT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[ADESC]", "[ADESC" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AQTMED]", "[AQTMED" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AMEDIDA]", "[AMEDIDA" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[ATPSERV]", "[ATPSERV" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AQTTPSERV]", "[AQTTPSERV" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AMODPAG]", "[AMODPAG" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AVRUNIT]", "[AVRUNIT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AVRTOT]", "[AVRTOT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[ATOTPER]", "[ATOTPER" . $codigoItem . "]", $clobLinhaSubstituida);
        
        //$clobLinhaSubstituida = str_replace("[AVRPRRECI]", "[AVRPRRECI" . $codigoItem . "]", $clobLinhaSubstituida);
        //$clobLinhaSubstituida = str_replace("[AVRRECI]", "[AVRRECI" . $codigoItem . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    private function retirarMnemonicosItensApuracaoFaturamento($clobLinha) {
        $clobLinhaSubstituida = str_replace("[AIND]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[ACOD]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AQNT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[ADESC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AQTMED]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AMEDIDA]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[ATPSERV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AQTTPSERV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AMODPAG]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AVRUNIT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[AVRTOT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[ATOTPER]", "", $clobLinhaSubstituida);
        
        //$clobLinhaSubstituida = str_replace("[AVRPRRECI]", "", $clobLinhaSubstituida);
        //$clobLinhaSubstituida = str_replace("[AVRRECI]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    public function gerarDocumentoParteDeItensModalidadePagamento($nrseqcontrato, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        $result = null;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[OIND]" no documento inteiro
        $posicaoInd = strpos($documento, "[OIND]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr" que antecede o "[OIND]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
            $posicaoPrimeiroLN = strpos($documento, "[/LN]", $posicaoInd);
            
            //posicao do primeiro "[LN]" que antecede o "[OIND]"
            $posicaoFechaLN = strrpos($documento, "[LN]", (($tamanhoDocumento - $posicaoPrimeiroLN) * -1));
            
            
            if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN !== false) {
                if ($posicaoPrimeiroTr < $posicaoPrimeiroLN) {
                    $result = "C";
                    
                    $posIni = $posicaoPrimeiroTr;
                    $posFin = $posicaoFechaTr;
                    $tagFecha = "</w:tr>";
                }
                else {
                    $result = "L";
                
                    //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                    $posFin = strpos($documento, "[/LN]", $posicaoInd);
                    
                    //posicao do primeiro "[LN]" que antecede o "[IND]"
                    $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                    
                    $tagFecha = "[/LN]";
                }
            }
            else if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN === false) {
                $result = "C";
                
                $posIni = $posicaoPrimeiroTr;
                $posFin = $posicaoFechaTr;
                $tagFecha = "</w:tr>";
            }
            else {
                $result = "L";
                
                //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                $posFin = strpos($documento, "[/LN]", $posicaoInd);
                
                //posicao do primeiro "[LN]" que antecede o "[IND]"
                $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "[/LN]";
            }
            
            $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
            $clobLinha = substr($documento, $posIni, $posFin - $posIni);
            $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
            
            $linhasCodigoItens = $this->getCodigoItensModalidadePagamentoContrato($nrseqcontrato, $nrorg);
            $qtdItens = count($linhasCodigoItens);
            
            foreach ($linhasCodigoItens as $linhaCodigoItem) {
                //coloca o índice nos mnemônicos dos itens
                $clobLinhaSubstituida = $this->colocarIndiceMnemonicoItensModalidadePagamento($linhaCodigoItem["IND"], $clobLinha);
                
                if ($result == "C") {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                    //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1||'</w:tr>';
                }
                else {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida;
                    //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1;
                }
            }
            
            if ($qtdItens > 0) {
                //retira os mnemônicos dos itens
                $clobLinhaSubstituida = $this->retirarMnemonicosItensModalidadePagamento($clobLinha);
                
                $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
            }
            
            $documento = str_replace($clobLinha.$tagFecha, "", $documento);
        }
        
        return $documento;
    }
    
    private function colocarIndiceMnemonicoItensModalidadePagamento($codigoItem, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[OIND]", "[OIND" . $codigoItem . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[OCOD]", "[OCOD" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OQNT]", "[OQNT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[ODESC]", "[ODESC" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OQTMED]", "[OQTMED" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OMEDIDA]", "[OMEDIDA" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OTPSERV]", "[OTPSERV" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OQTTPSERV]", "[OQTTPSERV" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OMODPAG]", "[OMODPAG" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OVRUNIT]", "[OVRUNIT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OVRTOT]", "[OVRTOT" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OTOTPER]", "[OTOTPER" . $codigoItem . "]", $clobLinhaSubstituida);
        
        //$clobLinhaSubstituida = str_replace("[OVRPRRECI]", "[OVRPRRECI" . $codigoItem . "]", $clobLinhaSubstituida);
        //$clobLinhaSubstituida = str_replace("[OVRRECI]", "[OVRRECI" . $codigoItem . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    private function retirarMnemonicosItensModalidadePagamento($clobLinha) {
        $clobLinhaSubstituida = str_replace("[OIND]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[OCOD]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OQNT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[ODESC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OQTMED]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OMEDIDA]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OTPSERV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OQTTPSERV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OMODPAG]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OVRUNIT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OVRTOT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[OTOTPER]", "", $clobLinhaSubstituida);
        
        //$clobLinhaSubstituida = str_replace("[OVRPRRECI]", "", $clobLinhaSubstituida);
        //$clobLinhaSubstituida = str_replace("[OVRRECI]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    public function getCodigoItensApuracaoFaturamentoContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->documentoFactory->getCodigoItensApuracaoFaturamentoContrato($params);
    }
    
    public function getCodigoItensModalidadePagamentoContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->documentoFactory->getCodigoItensModalidadePagamentoContrato($params);
    }
    
    public function getItensContrato($nrseqcontrato, $nrorg) {
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->documentoFactory->getItensContrato($params);
    }
    
    private function substituirDadosItensDocumento($item, $documento) {
        //$documento = str_replace("[IND".$item["IND"]."]", $item["IND"], $documento);
        $documento = str_replace("[IND".$item["IND"]."]", "", $documento);
        
        $documento = str_replace("[COD".$item["IND"]."]", $item["CODITEM"], $documento);
        $documento = str_replace("[QNT".$item["IND"]."]", $item["QTITEMCONTRATO"], $documento);
        $documento = str_replace("[DESC".$item["IND"]."]", str_replace("&", "&amp;", $item["NMITEMCONTRATO"]), $documento);
        $documento = str_replace("[QTMED".$item["IND"]."]", $item["QTMEDIDACONTR"], $documento);
        $documento = str_replace("[MEDIDA".$item["IND"]."]", str_replace("&", "&amp;", $item["DSTPMEDIDASERV"]), $documento);
        $documento = str_replace("[TPSERV".$item["IND"]."]", $item["IDUNIDTEMPOSERV"], $documento);
        $documento = str_replace("[QTTPSERV".$item["IND"]."]", $item["QTTEMPOSERV"], $documento);
        $documento = str_replace("[MODPAG".$item["IND"]."]", str_replace("&", "&amp;", $item["DSMODPGTO"]), $documento);
        $documento = str_replace("[VRUNIT".$item["IND"]."]", Utilities::formatarNumero($item["VRUNITITEMCONT"]), $documento);
        $documento = str_replace("[VRTOT".$item["IND"]."]", Utilities::formatarNumero($item["VRTOTITEMCONT"]), $documento);
        $documento = str_replace("[TOTPER".$item["IND"]."]", Utilities::formatarNumero($item["TOTPERIODO"]), $documento);

        $vrTotalSemDesconto = $item["VRUNITITEMCONT"] * $item["QTITEMCONTRATO"];
        if ($item["IDMULTMEDVRTOT"] == "S" && (float)$item["QTMEDIDACONTR"] > 0) {
            $vrTotalSemDesconto *= (float)$item["QTMEDIDACONTR"];
        }
        
        $vrDesconto = $vrTotalSemDesconto - $item["VRTOTITEMCONT"];
        if ($vrDesconto < 0) $vrDesconto = 0;
        $documento = str_replace("[VRDESC".$item["IND"]."]", Utilities::formatarNumero($vrDesconto), $documento);
        
        $vrAcrescimo = $item["VRTOTITEMCONT"] - $vrTotalSemDesconto;
        if ($vrAcrescimo < 0) $vrAcrescimo = 0;
        $documento = str_replace("[VRACRE".$item["IND"]."]", Utilities::formatarNumero($vrAcrescimo), $documento);
        
        $vrAjuste = $item["VRTOTITEMCONT"] - $vrTotalSemDesconto;
        $documento = str_replace("[VRAJUSTE".$item["IND"]."]", Utilities::formatarNumero($vrAjuste), $documento);
        
        $documento = str_replace("[VRSEMDESC".$item["IND"]."]", Utilities::formatarNumero($vrTotalSemDesconto), $documento);
        $documento = str_replace("[VRSEMAJUSTE".$item["IND"]."]", Utilities::formatarNumero($vrTotalSemDesconto), $documento);
        
        //$documento = str_replace("[VRPRRECI".$item["IND"]."]", $item[""], $documento);
        //$documento = str_replace("[VRRECI".$item["IND"]."]", $item[""], $documento);
        
        return $documento;
    }
    
    private function substituirDadosItensApuracaoFaturamentoDocumento($item, $documento) {
        //$documento = str_replace("[AIND".$item["IND"]."]", $item["IND"], $documento);
        $documento = str_replace("[AIND".$item["IND"]."]", "", $documento);
        
        $documento = str_replace("[ACOD".$item["IND"]."]", $item["CODITEM"], $documento);
        $documento = str_replace("[AQNT".$item["IND"]."]", $item["QTITEMCONTRATO"], $documento);
        $documento = str_replace("[ADESC".$item["IND"]."]", str_replace("&", "&amp;", $item["NMITEMCONTRATO"]), $documento);
        $documento = str_replace("[AQTMED".$item["IND"]."]", $item["QTMEDIDACONTR"], $documento);
        $documento = str_replace("[AMEDIDA".$item["IND"]."]", str_replace("&", "&amp;", $item["DSTPMEDIDASERV"]), $documento);
        $documento = str_replace("[ATPSERV".$item["IND"]."]", $item["IDUNIDTEMPOSERV"], $documento);
        $documento = str_replace("[AQTTPSERV".$item["IND"]."]", $item["QTTEMPOSERV"], $documento);
        $documento = str_replace("[AMODPAG".$item["IND"]."]", str_replace("&", "&amp;", $item["DSMODPGTO"]), $documento);
        $documento = str_replace("[AVRUNIT".$item["IND"]."]", Utilities::formatarNumero($item["VRUNITITEMCONT"]), $documento);
        $documento = str_replace("[AVRTOT".$item["IND"]."]", Utilities::formatarNumero($item["VRTOTITEMCONT"]), $documento);
        $documento = str_replace("[ATOTPER".$item["IND"]."]", Utilities::formatarNumero($item["TOTPERIODO"]), $documento);
        
        //$documento = str_replace("[AVRPRRECI".$item["IND"]."]", $item[""], $documento);
        //$documento = str_replace("[AVRRECI".$item["IND"]."]", $item[""], $documento);
        
        //$documento = str_replace("[AVRDESC".$item["IND"]."]", Utilities::formatarNumero($item["VRITEMSEMDESC"] - $item["VRTOTALITEMORCA"]), $documento);
        //$documento = str_replace("[AVRSEMDESC".$item["IND"]."]", Utilities::formatarNumero($item["VRITEMSEMDESC"]), $documento);
        
        return $documento;
    }
    
    private function substituirDadosItensModalidadePagamentoDocumento($item, $documento) {
        //$documento = str_replace("[OIND".$item["IND"]."]", $item["IND"], $documento);
        $documento = str_replace("[OIND".$item["IND"]."]", "", $documento);
        
        $documento = str_replace("[OCOD".$item["IND"]."]", $item["CODITEM"], $documento);
        $documento = str_replace("[OQNT".$item["IND"]."]", $item["QTITEMCONTRATO"], $documento);
        $documento = str_replace("[ODESC".$item["IND"]."]", str_replace("&", "&amp;", $item["NMITEMCONTRATO"]), $documento);
        $documento = str_replace("[OQTMED".$item["IND"]."]", $item["QTMEDIDACONTR"], $documento);
        $documento = str_replace("[OMEDIDA".$item["IND"]."]", str_replace("&", "&amp;", $item["DSTPMEDIDASERV"]), $documento);
        $documento = str_replace("[OTPSERV".$item["IND"]."]", $item["IDUNIDTEMPOSERV"], $documento);
        $documento = str_replace("[OQTTPSERV".$item["IND"]."]", $item["QTTEMPOSERV"], $documento);
        $documento = str_replace("[OMODPAG".$item["IND"]."]", str_replace("&", "&amp;", $item["DSMODPGTO"]), $documento);
        $documento = str_replace("[OVRUNIT".$item["IND"]."]", Utilities::formatarNumero($item["VRUNITITEMCONT"]), $documento);
        $documento = str_replace("[OVRTOT".$item["IND"]."]", Utilities::formatarNumero($item["VRTOTITEMCONT"]), $documento);
        $documento = str_replace("[OTOTPER".$item["IND"]."]", Utilities::formatarNumero($item["TOTPERIODO"]), $documento);
        
        //$documento = str_replace("[OVRPRRECI".$item["IND"]."]", $item[""], $documento);
        //$documento = str_replace("[OVRRECI".$item["IND"]."]", $item[""], $documento);
        
        //$documento = str_replace("[OVRDESC".$item["IND"]."]", Utilities::formatarNumero($item["VRITEMSEMDESC"] - $item["VRTOTALITEMORCA"]), $documento);
        //$documento = str_replace("[OVRSEMDESC".$item["IND"]."]", Utilities::formatarNumero($item["VRITEMSEMDESC"]), $documento);
        
        return $documento;
    }
    
    public function getItensAdicionaisContrato($nrseqcontrato, $nritemcontratopai, $nrorg) {
        $params = array(
            "NRSEQCONTRATO"     => $nrseqcontrato,
            "NRITEMCONTRATOPAI" => $nritemcontratopai,
            "NRORG"             => $nrorg
        );
        
        return $this->documentoFactory->getItensAdicionaisContrato($params);
    }
    
    public function retirarMnemonicosItens($clobLinha) {
        $clobLinhaSubstituida = str_replace("[IND]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[COD]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QNT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTMED]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[MEDIDA]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPSERV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[QTTPSERV]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[MODPAG]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRUNIT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRTOT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TOTPER]", "", $clobLinhaSubstituida);
        
        $clobLinhaSubstituida = str_replace("[VRDESC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRACRE]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRAJUSTE]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRSEMDESC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRSEMAJUSTE]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    public function substituirDadosItensAdicionaisDocumento($item, $itemAdicional, $documento) {
        //$documento = str_replace("[INDAD" . $item["NRITEMOCONTRATO"] . $itemAdicional["IND"] . "]", $itemAdicional["IND"], $documento);
        $documento = str_replace("[INDAD" . $item["NRITEMOCONTRATO"] . $itemAdicional["IND"] . "]", "", $documento);
        
        $documento = str_replace("[DESCAD" . $item["NRITEMOCONTRATO"] . $itemAdicional["IND"] . "]", str_replace("&", "&amp;", $itemAdicional["NMITEMCONTRATO"]), $documento);
        $documento = str_replace("[QNTAD" . $item["NRITEMOCONTRATO"] . $itemAdicional["IND"] . "]", $itemAdicional["QTITEMCONTRATO"], $documento);
        $documento = str_replace("[VRUNITAD" . $item["NRITEMOCONTRATO"] . $itemAdicional["IND"] . "]", Utilities::formatarNumero($itemAdicional["VRUNITITEMCONT"]), $documento);
        $documento = str_replace("[MEDIDAAD" . $item["NRITEMOCONTRATO"] . $itemAdicional["IND"] . "]", str_replace("&", "&amp;", $itemAdicional["DSTPMEDIDASERV"]), $documento);
        $documento = str_replace("[QTMEDAD" . $item["NRITEMOCONTRATO"] . $itemAdicional["IND"] . "]", $itemAdicional["QTMEDIDACONTR"], $documento);
        $documento = str_replace("[VRTOTAD" . $item["NRITEMOCONTRATO"] . $itemAdicional["IND"] . "]", Utilities::formatarNumero($itemAdicional["VRTOTITEMCONT"]), $documento);
        
        return $documento;
    }
    
    /**
    *  Substitui o valor pego das propostas da Intranet  
    */
    public function substituirCodigoPropostaIntranet($codigoPropostaIntranet, $templateProcessor) {
        $templateProcessor->setValue("[CDPROPINTRANET]", $codigoPropostaIntranet);
    }
    
    public function getCaminhoLogotipoParceiro($nrparcnegocio, $nrorg) {
        $url = $this->getUrlFileServerLogotipoParceiro($nrparcnegocio, $nrorg);
        if (!empty($url)) {
            $logo = $this->smdocumentos->getDocumentoFileServer($this->fileServerBaseUrl . $url, $nrorg);
            if ($logo !== false) {
                $posPonto = strrpos($url, ".");
                $extensaoArquivo = substr($url, $posPonto);
                
                $caminhoLogotipo = $this->tempDir . DIRECTORY_SEPARATOR ."logo_cliente_" . $nrparcnegocio . $extensaoArquivo;
                file_put_contents($caminhoLogotipo, $logo);
                
                return $caminhoLogotipo;
            }
            else {
                return null;
            }
        }
        else {
            return null;
        }
    }
    
    public function getUrlFileServerLogotipoParceiro($nrparcnegocio, $nrorg) {
        $params = array(
            "NRPARCNEGOCIO" => $nrparcnegocio,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->documentoFactory->getUrlFileServerLogotipoParceiro($params);
        return $dataset[0]["DSDIRLOGOMARCA"];
    }
    
    public function substituirLogotipoDocumento($caminhoLogotipo, $templateProcessor, $tamanhoLogo = null) {
        $imagem = array(
            "src" => $caminhoLogotipo,
            //"swh" => "400"
        );

        if ($tamanhoLogo)
            $imagem["swh"] = $tamanhoLogo;
        
        $templateProcessor->setImg("[LOGOTIPO]", $imagem);
    }
    
    /**
     * Retorna os códigos de formação de preço de um contrato 
    **/
    public function getCodigoFormacaoPrecoContrato($nrorg, $nrseqcontrato){
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->documentoFactory->getCodigoFormacaoPrecoContrato($params);
    }
    
    /**
     * Adiciona o indice de mnemônicos para cada formação de preço de um contrato
    **/
    public function colocarIndiceMnemonicoFormacoesPreco($indiceFormacao, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDFP]", "[INDFP" . $indiceFormacao . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCFP]", "[DESCFP" . $indiceFormacao . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESCSRVFP]", "[DESCSRVFP" . $indiceFormacao . "]", $clobLinhaSubstituida);

        return $clobLinhaSubstituida;
    }
    
    /**
     * Retira os mnemônicos de formação de preço que não foram usados no documento
    **/
    public function retirarMnemonicosFormacoesPreco($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDFP]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCFP]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESCSRVFP]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }

    /**
     * Substitui os mnemônicos de formação de preço do contrato
    **/
    public function substituirDadosFormacoesPrecoDocumento($formacao, $clobLinhaSubstituida) {
        $clobLinhaSubstituida = str_replace("[INDFP".$formacao["IND"]."]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESCFP".$formacao["IND"]."]", str_replace("&", "&amp;", $formacao["NMFORMPRECOSERVICO"]), $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[DESCSRVFP".$formacao["IND"]."]", str_replace("&", "&amp;", $formacao["NMSERVORCA"]), $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Retorna os dados dos itens de uma formação de preço
    **/
    public function getItensFormacaoPrecoContrato($nrformprecoservico, $nrorg){
        $params = array(
            "NRFORMPRECOSERVICO" => $nrformprecoservico,
            "NRORG"              => $nrorg
        );
        
        return $this->documentoFactory->getItensFormacaoPrecoContrato($params);
    }
    
    /**
     * formata os itens de formação de preço colocando o IDITEMCALCULO 
     * como indice
    **/
    public function formatarItensFormacaoPrecoPorIdentificador($itensFormacaoPreco) {
        $novosItens = array();
        foreach ($itensFormacaoPreco as $item) {
            $novosItens[$item["IDITEMCALCULO"]] = $item;
        }
        
        return $novosItens;
    }
    
    /**
     * Retira os mnemônicos que não foram substituidos na formaçao de preço
    **/
    public function retirarMnemonicosItensFormacaoPrecoPorIdentificacao($clobLinhaSubstituida) {
        $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
        $posicaoId = strpos($clobLinhaSubstituida, "[ID_");
        while ($posicaoId !== false) {
            
            $posicaoFechaTr = strpos($clobLinhaSubstituida, "</w:tr>", $posicaoId);
            $posicaoPrimeiroTr = strrpos($clobLinhaSubstituida, "<w:tr ", (($tamanhoClobLinhaSubstituida - $posicaoFechaTr) * -1));
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($clobLinhaSubstituida, "<w:tr>", (($tamanhoClobLinhaSubstituida - $posicaoFechaTr) * -1));
            }
            
            $posIni = $posicaoPrimeiroTr;
            $posFin = $posicaoFechaTr;
            $tagFecha = "</w:tr>";

            $clobLinha = substr($clobLinhaSubstituida, $posIni, $posFin - $posIni);
            $clobLinha .= $tagFecha;
            
            $clobLinhaSubstituida = str_replace($clobLinha, "", $clobLinhaSubstituida);
            $tamanhoClobLinhaSubstituida = strlen($clobLinhaSubstituida);
            $posicaoId = strpos($clobLinhaSubstituida, "[ID_");
        }
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Verifica na tabela de alterações da formação de preço se há a formação procurada 
    **/
    public function getItensFormacaoAuxiliarProposta($nrorg, $nrformprecoservico, $nritemorcamento, $nrseqorcamento){
        $params = array(
            "NRSEQORCAMENTO"     => $nrseqorcamento,
            "NRITEMORCAMENTO"    => $nritemorcamento,
            "NRFORMPRECOSERVICO" => $nrformprecoservico,
            "NRORG"              => $nrorg
        );
        
        return $this->documentoFactory->getItensFormacaoAuxiliarProposta($params);
    }
    
    /**
     * Verifica na tabela de alterações da formação de preço se há a formação procurada 
    **/
    public function getItensFormacaoAuxiliarContrato($nrorg, $nrformprecoservico, $nritemocontrato, $nrseqcontrato){
        $params = array(
            "NRSEQCONTRATO"      => $nrseqcontrato,
            "NRITEMOCONTRATO"    => $nritemocontrato,
            "NRFORMPRECOSERVICO" => $nrformprecoservico,
            "NRORG"              => $nrorg
        );
        
        return $this->documentoFactory->getItensFormacaoAuxiliarContrato($params);
    }

    /**
     * Monta a parte de itens das formações de preço de um contrato
    **/
    public function gerarDocumentoParteDeItensFormacaoPrecoPorIdentificacao($nrorg, $formprecoservico, $clobLinhaSubstituida) {
        $itensFormacaoPreco = array();
        $itensFormacaoPreco = $this->getItensFormacaoAuxiliarProposta($nrorg, $formprecoservico["NRFORMPRECOSERVICO"], $formprecoservico["NRITEMORCAMENTO"], $formprecoservico["NRSEQORCAMENTO"]);
        
        if(count($itensFormacaoPreco) <= 0){
            $itensFormacaoPreco = $this->getItensFormacaoAuxiliarContrato($nrorg, $formprecoservico["NRFORMPRECOSERVICO"], $formprecoservico["NRITEMOCONTRATO"], $formprecoservico["NRSEQCONTRATO"]);
            
            if(count($itensFormacaoPreco) <= 0)    
                $itensFormacaoPreco = $this->getItensFormacaoPrecoContrato($formprecoservico["NRFORMPRECOSERVICO"], $nrorg);
        }
        
        $itensFormacaoPreco = $this->formatarItensFormacaoPrecoPorIdentificador($itensFormacaoPreco);
        $identificadores = array_keys($itensFormacaoPreco);
        foreach ($identificadores as $identificador) {
            
            $campos = array(
                "_DESC"     => array(
                    "NOME"            => "NMELEMENTO",
                    "FORMATAR_NUMERO" => false
                ),
                "_VRPERC"   => array(
                    "NOME"            => "VRPERCCALCULO",
                    "FORMATAR_NUMERO" => true
                ),
                "_VRTOT"    => array(
                    "NOME"            => "VRTOTAL",
                    "FORMATAR_NUMERO" => true
                ),
                "_QTMEDIDA" => array(
                    "NOME"            => "QTMEDIDACALC",
                    "FORMATAR_NUMERO" => true
                ),
                "_VRUNIT"   => array(
                    "NOME"            => "VRUNITARIO",
                    "FORMATAR_NUMERO" => true
                ),
                "_MEDIDA"    => array(
                    "NOME"            => "DSTPMEDIDASERV",
                    "FORMATAR_NUMERO" => false
                )
            );
            
            foreach ($campos as $chave => $campo) {
                $mnemonico = "[ID_" . $identificador . $chave . "]";
                $posicao = strpos($clobLinhaSubstituida, $mnemonico);
                
                if ($posicao !== false) {
                    $valor = substr($clobLinhaSubstituida, $posicao, strlen($mnemonico));
                    $substituicao = $itensFormacaoPreco[$identificador][$campo["NOME"]];
                    
                    if ($campo["FORMATAR_NUMERO"] && !empty($substituicao)) {
                        $substituicao = Utilities::formatarNumero($substituicao);
                    }
                    
                    $clobLinhaSubstituida = str_replace($valor, $substituicao, $clobLinhaSubstituida);
                }
            }
        }
        
        $clobLinhaSubstituida = $this->retirarMnemonicosItensFormacaoPrecoPorIdentificacao($clobLinhaSubstituida);
        return $clobLinhaSubstituida;
    }
    
    /**
     * Gera a parte de formação de preço do documento do contrato
    **/
    public function gerarDocumentoParteFormacaoPreco($nrorg, $nrseqcontrato, $documento){
        $posicaoIni = -1;
        $posicaoFim = -1;
        
        $tamanhoDocumento = strlen($documento);
        $posicaoInd = strpos($documento, "[INDFP]");
        
        if($posicaoInd != false){
            $quebralinha = strpos($documento, "[QBRPAGFP]");
            if ($quebralinha !== false){
                $pageBreak = '<w:p>
                                <w:r>
                                    <w:br w:type="page"/>
                                </w:r>
                             </w:p>';
                             
                $documento = str_replace("[QBRPAGFP]", "", $documento);
            } else {
                $pageBreak = '<w:br/>';
            }
            
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            $posicaoAbreTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoAbreTr) {
                $posicaoAbreTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            if($posicaoAbreTr != false){
                $posicaoFim = strpos($documento, "</w:tbl>", $posicaoAbreTr);
                $posicaoIni  = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posicaoFim) * -1));
                $tagFecha = "</w:tbl>";
                
                $clobInicial = substr($documento, 0, $posicaoFim + strlen($tagFecha));
                $clobLinha   = substr($documento, $posicaoIni, $posicaoFim - $posicaoIni);
                $clobFinal   = substr($documento, $posicaoFim, $tamanhoDocumento);
                
                $formsprecoscontrato = $this->getCodigoFormacaoPrecoContrato($nrorg, $nrseqcontrato);
                $qtdcodigoform = count($formsprecoscontrato);

                foreach ($formsprecoscontrato as $formprecocontrato) {
                    $clobLinhaSubstituida = $this->colocarIndiceMnemonicoFormacoesPreco($formprecocontrato["IND"], $clobLinha);
                    $clobLinhaSubstituida = $this->substituirDadosFormacoesPrecoDocumento($formprecocontrato, $clobLinhaSubstituida);
                    $clobLinhaSubstituida = $this->gerarDocumentoParteDeItensFormacaoPrecoPorIdentificacao($nrorg, $formprecocontrato, $clobLinhaSubstituida);
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha . $pageBreak;
                }
                
                if ($qtdcodigoform > 0) {
                    $clobLinhaSubstituida = $this->retirarMnemonicosFormacoesPreco($clobLinha);
                    $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                $documento = str_replace($clobLinha.$tagFecha, "", $documento);
            }
        }
        return $documento;
    }
    
      /**
     * Monta os mnemônicos com índice da parte de itens e adicionais no documento, para serem substituidos pelos valores depois.
     */
    public function gerarDocumentoParteDeHoraExtra($nrseqcontrato, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        $result = null;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[IND]" no documento inteiro
        $posicaoInd = strpos($documento, "[INDHRX]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr" que antecede o "[IND]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            if ($posicaoPrimeiroTr !== false) {
                $posIni = $posicaoPrimeiroTr;
                $posFin = $posicaoFechaTr;
                $tagFecha = "</w:tr>";
                
                $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
                $clobLinha = substr($documento, $posIni, $posFin - $posIni);
                $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
                
                $linhasCodigoItens = $this->getCodigoItensContrato($nrseqcontrato, $nrorg);
                $qtdItens = count($linhasCodigoItens);
                
                foreach ($linhasCodigoItens as $linhaCodigoItem) {
                    //coloca o índice nos mnemônicos dos itens
                    $clobLinhaSubstituida = $this->colocarIndiceMnemonicoHrExtra($linhaCodigoItem["IND"], $clobLinha);
                    
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                }
                
                if ($qtdItens > 0) {
                    //retira os mnemônicos dos itens
                    //$clobLinhaSubstituida = $this->retirarMnemonicosItens($clobLinha);
                    
                    $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                $documento = str_replace($clobLinha.$tagFecha, "", $documento);
            }
        }
        
        return $documento;
    }
    
    
     /**
     * Coloca o índice nos mnemônicos das horas Extras da proposta.
     */
    private function colocarIndiceMnemonicoHrExtra($codigoItem, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDHRX]", "[INDHRX" . $codigoItem . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCHRX]", "[DESCHRX" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRHORA]", "[VRHORA" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRHORAEX25]", "[VRHORAEX25" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRHORAEX50]", "[VRHORAEX50" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRHORAEX70]", "[VRHORAEX70" . $codigoItem . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRHORAEX100]", "[VRHORAEX100" . $codigoItem . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    
    /**
     *Limpa os dados de Hora Extra do documento
     */
    
    public function retirarMnemonicosHrExtra($clobLinha) {
        
        $clobLinhaSubstituida = str_replace("[INDHRX]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[DESCHRX]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRHORA]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRHORAEX25]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRHORAEX50]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRHORAEX70]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[VRHORAEX100]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Subustitui os mnemonicos de Hora Extra do documento. 
     */

    private function substituirDadosHrExtraDocumento($item, $documento) {
    
        $documento = str_replace("[INDHRX".$item["IND"]."]", "", $documento);
        $documento = str_replace("[DESCHRX".$item["IND"]."]", str_replace("&", "&amp;", $item["NMITEMCONTRATO"]), $documento);
        
        $vrhora = $item["VRTOTITEMCONT"]/220;
       
        /*25*/
        $vrHrExtra25 = $vrhora *1.25;
        $vrHrExtra25 = Utilities::formatarNumero($vrHrExtra25);
     
        /* 50% */
        $vrHrExtra50 = $vrhora*1.5;
        $vrHrExtra50 = Utilities::formatarNumero($vrHrExtra50);
       
       /* 70% */
        $vrHrExtra70 = $vrhora*1.7;
        $vrHrExtra70 = Utilities::formatarNumero($vrHrExtra70);
       
       /* 100% */
        $vrHrExtra100 = $vrhora*2;
        $vrHrExtra100 = Utilities::formatarNumero($vrHrExtra100);
         
        $documento = str_replace("[VRHORA".$item["IND"]."]", $vrhora, $documento);
        $documento = str_replace("[VRHORAEX25".$item["IND"]."]", $vrHrExtra25, $documento);
        $documento = str_replace("[VRHORAEX50".$item["IND"]."]", $vrHrExtra50, $documento);
        $documento = str_replace("[VRHORAEX70".$item["IND"]."]", $vrHrExtra70, $documento);
        $documento = str_replace("[VRHORAEX100".$item["IND"]."]", $vrHrExtra100, $documento);
        
        return $documento;
    }
    /**
     * Valida se o documento mostrará os itens da proposta e se eles existem
    **/
    public function validarExistenciaItensDocumento($documento, $nrseqcontrato, $nrorg) {
        $posicaoInd = strpos($documento, "[IND]");
        if ($posicaoInd !== false) {
            $itens = $this->getItensContrato($nrseqcontrato, $nrorg);
            return count($itens) > 0;
        }
        
        return true;
    }
    
    /**
     * Monta a parte de Assinaturas do contrato.
     */
    private function gerarDocumentoAssinaturas($nrseqcontrato, $nrorg, $documento) {
        $posIni = -1;
        $posFin = -1;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[INDSIGN]" no documento inteiro
        $posicaoIndSign = strpos($documento, "[INDSIGN]");
        
        if ($posicaoIndSign !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoIndSign
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoIndSign);
            
            //posicao do primeiro "<w:tr>" que antecede o "[INDSIGN]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            if ($posicaoPrimeiroTr !== false) {
                //posicao do primeiro "</w:tbl>" no documento, contando a partir de $posicaoIndSign
                $posFin = strpos($documento, "</w:tbl>", $posicaoIndSign);
                
                //posicao do primeiro "<w:tbl>" que antecede o "[INDSIGN]"
                $posIni = strrpos($documento, "<w:tbl>", (($tamanhoDocumento - $posFin) * -1));
                
                $tagFecha = "</w:tbl>";
            
            
                $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
                $clobLinha = substr($documento, $posIni, $posFin - $posIni);
                $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
                
                $assinaturas = $this->getAssinaturasContrato($nrseqcontrato, $nrorg);
                $qtdItens = count($assinaturas);
                
                foreach ($assinaturas as $assinatura) {

                    //coloca os indices nos mnemônicos das assinaturas do contrato
                    $clobLinhaSubstituida = $this->colocarIndiceAssinaturasContrato($assinatura["IND"], $clobLinha);
                    
                    //monta a parte das assinaturas do contrato
                    $clobLinhaSubstituida = $this->gerarDocumentoAssinaturasContrato($nrorg, $assinatura, $clobLinhaSubstituida);
                    
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                }
                
                if ($qtdItens > 0) {
                    //Retira os mnemônicos das assinaturas do contrato do documento
                    $clobLinhaSubstituida = $this->retirarMnemonicosAssinaturasContrato($clobLinha);
                    
                    $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
                }
                
                $documento = str_replace($clobLinha.$tagFecha, "", $documento);
            }
        }
        
        return $documento;
    }
    
    /**
     * Retorna as assinaturas do contrato
     */
    private function getAssinaturasContrato($nrseqcontrato, $nrorg) {
        
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        return $this->documentoFactory->getAssinaturasContrato($params);
    }
    
      /**
     * coloca os indices nos mnemônicos das assinaturas do contrato
     */
    private function colocarIndiceAssinaturasContrato($indice, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDSIGN]",   "[INDSIGN"   . $indice . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[NMSIGNAT]",  "[NMSIGNAT"  . $indice . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[CPFSIGNAT]", "[CPFSIGNAT" . $indice . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPASSINAT]", "[TPASSINAT" . $indice . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPSIGNAT]",  "[TPSIGNAT"  . $indice . "]", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Monta a parte das assinaturas do contrato
     */
    private function gerarDocumentoAssinaturasContrato($nrorg, $assinatura, $clobLinhaSubstituida) {
        
        $nomeSignatario = $assinatura["NMSIGNATARIO"];
        $cpfSignatario  = Utilities::formatarCpfCnpj($assinatura["NRCPFSIGNATARIO"]);
        $tipoAssinatura = $assinatura["IDTIPOASSINATURA"];
        $tipoSignatario = $assinatura["IDTESTEMUNHA"];
        
        if($tipoAssinatura == "E"){
            $tipoAssinatura = "Eletrônica";
        } else {
            $tipoAssinatura = "Digital";
        }
        
        if($tipoSignatario == "S"){
            $tipoSignatario = "Testemunha";
        } else {
            $tipoSignatario = "Signatário";
        }
        
        $clobLinhaSubstituida = str_replace("[INDSIGN"   . $assinatura["IND"] . "]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[NMSIGNAT"  . $assinatura["IND"] . "]", $nomeSignatario, $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[CPFSIGNAT" . $assinatura["IND"] . "]", $cpfSignatario, $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPASSINAT" . $assinatura["IND"] . "]", $tipoAssinatura, $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPSIGNAT"  . $assinatura["IND"] . "]", $tipoSignatario, $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    
    }
    
    /**
     * Retira os mnemônicos das assinaturas do contrato do documento
     */
    private function retirarMnemonicosAssinaturasContrato($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDSIGN]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[NMSIGNAT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[CPFSIGNAT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPASSINAT]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[TPSIGNAT]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    /**
     * Substitui os dados de alteração contratual
     */
    public function substituirDadosAlteracaoContratual($nrseqcontrato, $nrorg, $documento) {
        
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->documentoFactory->getDadosAlteracaoContratual($params);
        
        $altContratual = (count($dataset) > 0) ? $dataset[0] : null;
        
        if (!empty($altContratual["DSALTERACONTRATUAIS"])) {
            
            $documento =  str_replace("[INIALTERCONTR]", "", $documento);
            $documento =  str_replace("[ALTERCONTR]", $altContratual["DSALTERACONTRATUAIS"], $documento);
            $documento =  str_replace("[FIMALTERCONTR]", "", $documento);
        }
        else{
            
            $posicaoIniAltContr = strpos($documento, "[INIALTERCONTR]");
            $posicaoFimAltContr = strpos($documento, "[FIMALTERCONTR]");
            
            if (($posicaoIniAltContr != false) && ($posicaoFimAltContr != false)) {
            
                $posicaoFimAltContr = ($posicaoFimAltContr - $posicaoIniAltContr)+15;
                
                $AlteraContr = substr($documento, $posicaoIniAltContr, $posicaoFimAltContr);
                
                $documento =  str_replace($AlteraContr, "", $documento);
            }
        }
            
        return $documento;
    }
    
    
    public function gerarDocumentoRescisaoContr($nrseqcontrato, $nrorg, $documento, $listaContr) {
        $posIni = -1;
        $posFin = -1;
        $result = null;
        
        $tamanhoDocumento = strlen($documento);
        
        //posicao do primeiro "[INDR]" no documento inteiro
        $posicaoInd = strpos($documento, "[INDR]");
        
        if ($posicaoInd !== false) {
            //posicao do primeiro "</w:tr>" no documento, contando a partir de $posicaoInd
            $posicaoFechaTr = strpos($documento, "</w:tr>", $posicaoInd);
            
            //posicao do primeiro "<w:tr" que antecede o "[INDR]"
            $posicaoPrimeiroTr = strrpos($documento, "<w:tr ", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            
            if (!$posicaoPrimeiroTr) {
                $posicaoPrimeiroTr = strrpos($documento, "<w:tr>", (($tamanhoDocumento - $posicaoFechaTr) * -1));
            }
            
            //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
            $posicaoPrimeiroLN = strpos($documento, "[/LN]", $posicaoInd);
            
            //posicao do primeiro "[LN]" que antecede o "[INDR]"
            $posicaoFechaLN = strrpos($documento, "[LN]", (($tamanhoDocumento - $posicaoPrimeiroLN) * -1));
            
            
            if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN !== false) {
                if ($posicaoPrimeiroTr < $posicaoPrimeiroLN) {
                    $result = "C";
                    
                    $posIni = $posicaoPrimeiroTr;
                    $posFin = $posicaoFechaTr;
                    $tagFecha = "</w:tr>";
                }
                else {
                    $result = "L";
                
                    //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                    $posFin = strpos($documento, "[/LN]", $posicaoInd);
                    
                    //posicao do primeiro "[LN]" que antecede o "[INDR]"
                    $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));
                    
                    $tagFecha = "[/LN]";
                }
            }
            else if ($posicaoPrimeiroTr !== false && $posicaoPrimeiroLN === false) {
                $result = "C";
                
                $posIni = $posicaoPrimeiroTr;
                $posFin = $posicaoFechaTr;
                $tagFecha = "</w:tr>";
            }
            else {
                $result = "L";
                
                //posicao do primeiro "[/LN]" no documento, contando a partir de $posicaoInd
                $posFin = strpos($documento, "[/LN]", $posicaoInd);
                
                //posicao do primeiro "[LN]" que antecede o "[INDR]"
                $posIni = strrpos($documento, "[LN]", (($tamanhoDocumento - $posFin) * -1));

                $tagFecha = "[/LN]";
            }
            
            $clobInicial = substr($documento, 0, $posFin + strlen($tagFecha));
            $clobLinha = substr($documento, $posIni, $posFin - $posIni);
            $clobFinal = substr($documento, $posFin, $tamanhoDocumento);
            
            $contratosResc = $this->getcontratosRescindidos($nrseqcontrato, $nrorg, $listaContr);
            $qtdItens = count($contratosResc);
            
            foreach ($contratosResc as $contResc) {
                //coloca o índice nos mnemônicos dos itens
                $clobLinhaSubstituida = $this->colocarIndiceMnemonicoContratosRescindidos($contResc["IND"], $clobLinha);
                
                if ($result == "C") {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida . $tagFecha;
                    //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1||'</w:tr>';
                }
                else {
                    $clobInicial = $clobInicial . $clobLinhaSubstituida;
                    //V_CLOBINICIAL := V_CLOBINICIAL||V_CLOBLINHA1;
                }
            }
            
            if ($qtdItens > 0) {
                //retira os mnemônicos dos itens
                $clobLinhaSubstituida = $this->retirarMnemonicosContratosRescindidos($clobLinha);
                
                $documento = $clobInicial . substr($clobFinal, strlen($tagFecha));
            }
            
            $documento = str_replace($clobLinha.$tagFecha, "", $documento);
            $documento = str_replace($clobLinha.$tagFecha, "", $documento);
        }
        
        return $documento;
    }
    
    public function getcontratosRescindidos($nrseqcontrato, $nrorg, $listaContr) {
        $params = array(
            "NRORG"  => $nrorg
        );
        
        return $this->documentoFactory->getcontratosRescindidos($params, $listaContr);
    }
    
    private function colocarIndiceMnemonicoContratosRescindidos($codContr, $clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDR]", "[INDR" . $codContr . "]", $clobLinha);
        $clobLinhaSubstituida = str_replace("[CDCONTRRESC]", "[CDCONTRRESC" . $codContr . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[REFECONTRRESC]", "[REFECONTRRESC" . $codContr . "]", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[CDORCACONTRESC]", "[CDORCACONTRESC" . $codContr . "]", $clobLinhaSubstituida);

        
        return $clobLinhaSubstituida;
    }
    
    private function retirarMnemonicosContratosRescindidos($clobLinha) {
        $clobLinhaSubstituida = str_replace("[INDR]", "", $clobLinha);
        $clobLinhaSubstituida = str_replace("[CDCONTRRESC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[REFECONTRRESC]", "", $clobLinhaSubstituida);
        $clobLinhaSubstituida = str_replace("[CDORCACONTRESC]", "", $clobLinhaSubstituida);
        
        return $clobLinhaSubstituida;
    }
    
    
    private function substituirDadosContratosRescindidos($nrseqcontrato, $nrorg, $documento, $listaContr) {
        
        $contratos = $this->getcontratosRescindidos($nrseqcontrato, $nrorg, $listaContr);
        foreach($contratos as $contr){

            $documento = str_replace("[INDR".$contr["IND"]."]", "", $documento);
            
            $documento = str_replace("[CDCONTRRESC".$contr["IND"]."]", $contr["NRCONTRATOFCT"], $documento);
            $documento = str_replace("[REFECONTRRESC".$contr["IND"]."]", $contr["DSREFERENCIACONT"], $documento);
            $documento = str_replace("[CDORCACONTRESC".$contr["IND"]."]", $contr["NRORCAMENTO"], $documento);
        }

        return $documento;
    }
    
    
     /**
     * Substitui os dados de rescisão do contrato
     */
    public function gerarDocumentoDadosRescisao($nrseqcontrato, $nrorg, $documento) {
        
        $params = array(
            "NRSEQCONTRATO" => $nrseqcontrato,
            "NRORG"         => $nrorg
        );
        
        $dataset = $this->documentoFactory->getDadosRescisao($params);
        
        $rescContrato = (count($dataset) > 0) ? $dataset[0] : null;
        
        if ($rescContrato != null) {
            if ($rescContrato["VRPENDENCIAFIN"] != null) { 
            
                $vrPendenciaFin = Utilities::formatarNumero($rescContrato["VRPENDENCIAFIN"]);
            }
            else {
                $vrPendenciaFin =  Utilities::formatarNumero(0);
            }
            
            $documento =  str_replace("[VRPENDFATRESC]", $vrPendenciaFin, $documento);
            $documento =  str_replace("[DTRESCISAO]", $rescContrato["DTRECISAOCONTR"], $documento);
            $documento =  str_replace("[DTPEDRESCISAO]", $rescContrato["DTPEDRESCISAOCONTR"], $documento);
            $documento =  str_replace("[JUSTIFICARESC]", $rescContrato["DSJUSTIFICAENCERRA"], $documento);
        }

        return $documento;
    }
    
}

?>